from .. import context
import json
from ..input_helpers import get_org_from_input_or_ctx, strip_none
import agilicus
import pem
from ..output.table import (
    spec_column,
    format_table,
    metadata_column,
    status_column,
    subtable,
    column,
)


def add_certificate(ctx, labels, certificate, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id

    results = []
    for certificate in pem.parse_file(certificate):
        spec = agilicus.TrustedCertificateSpec(str(certificate), **kwargs)
        if labels:
            spec.labels = [
                agilicus.TrustedCertificateLabelName(label) for label in labels
            ]
        req = agilicus.TrustedCertificate(spec=spec)

        try:
            created_cert = apiclient.trusted_certs_api.create_trusted_cert(req)
            results.append(created_cert)
        except Exception as exc:
            if exc.status == 409:
                body = exc.body
                if not body:
                    raise
                result = json.loads(body)
                guid = agilicus.find_guid(result)
                req = apiclient.trusted_certs_api.get_trusted_cert(guid, org_id=org_id)
                req.spec.certificate = str(certificate)
                if not req.spec.labels:
                    req.spec.labels = []
                for label in labels:
                    req.spec.labels.append(agilicus.TrustedCertificateLabelName(label))
                updated_cert = apiclient.trusted_certs_api.replace_trusted_cert(
                    req.metadata.id,
                    req,
                )
                results.append(updated_cert)
    return results


def get_certificate(ctx, certificate_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id

    kwargs = strip_none(kwargs)
    return apiclient.trusted_certs_api.get_trusted_cert(
        certificate_id,
        **kwargs,
    )


def delete_certificate(ctx, certificate_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id

    kwargs = strip_none(kwargs)
    return apiclient.trusted_certs_api.delete_trusted_cert(
        certificate_id,
        **kwargs,
    )


def add_label(ctx, label, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id

    labelName = agilicus.TrustedCertificateLabelName(label)
    spec = agilicus.TrustedCertificateLabelSpec(labelName, **kwargs)
    req = agilicus.TrustedCertificateLabel(spec=spec)
    return apiclient.trusted_certs_api.create_label(req)


def add_bundle(ctx, bundle, label=None, label_org_id=None, exclude=None, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id
    if label_org_id is None:
        label_org_id = org_id
    bundle_label = None
    if label is not None:
        label_name = agilicus.TrustedCertificateLabelName(label)
        label_spec = agilicus.TrustedCertificateLabelSpec(
            name=label_name, org_id=label_org_id
        )
        bundle_label = agilicus.TrustedCertificateBundleLabel(label=label_spec)
        if exclude is not None:
            bundle_label.exclude = exclude

    bundle_name = agilicus.TrustedCertificateBundleName(bundle)
    spec = agilicus.TrustedCertificateBundleSpec(bundle_name, **kwargs)
    if bundle_label:
        spec.labels = []
        spec.labels.append(bundle_label)
    req = agilicus.TrustedCertificateBundle(spec=spec)
    return apiclient.trusted_certs_api.create_bundle(req)


def add_bundle_label(ctx, bundle_id, label, label_org_id=None, exclude=None, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id
    if label_org_id is None:
        label_org_id = org_id

    bundle = get_bundle(ctx, bundle_id, **kwargs)
    bundle_label = None
    label_name = agilicus.TrustedCertificateLabelName(label)
    label_spec = agilicus.TrustedCertificateLabelSpec(
        name=label_name, org_id=label_org_id
    )
    bundle_label = agilicus.TrustedCertificateBundleLabel(label=label_spec)
    if exclude is not None:
        bundle_label.exclude = exclude

    if not bundle.spec.labels:
        bundle.spec.labels = []

    bundle.spec.labels.append(bundle_label)
    return apiclient.trusted_certs_api.replace_bundle(bundle_id, bundle)


def delete_bundle(ctx, bundle_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id
    return apiclient.trusted_certs_api.delete_bundle(bundle_id, **kwargs)


def list_bundles(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id

    kwargs = strip_none(kwargs)
    return apiclient.trusted_certs_api.list_bundles(
        **kwargs,
    ).trusted_cert_bundles


def get_bundle(ctx, bundle_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id

    kwargs = strip_none(kwargs)
    return apiclient.trusted_certs_api.get_bundle(
        bundle_id,
        **kwargs,
    )


def format_bundles(ctx, bundles):
    label_columns = [
        column("label"),
    ]
    columns = [
        metadata_column("id"),
        metadata_column("created"),
        spec_column("org_id"),
        spec_column("name"),
        subtable(ctx, "spec.labels", label_columns),
    ]
    return format_table(ctx, bundles, columns)


def list_labels(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id

    kwargs = strip_none(kwargs)
    return apiclient.trusted_certs_api.list_labels(
        **kwargs,
    ).trusted_cert_labels


def list_certificates(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs.pop("org_id", None)

    kwargs = strip_none(kwargs)
    query_results = apiclient.trusted_certs_api.list_trusted_certs(
        org_id=org_id, **kwargs
    ).trusted_certs
    return query_results


def format_labels(ctx, labels):
    columns = [
        metadata_column("created"),
        spec_column("org_id"),
        spec_column("name"),
    ]
    return format_table(ctx, labels, columns)


def format_certificates_as_text(ctx, certificates):
    columns = [
        metadata_column("id"),
        metadata_column("created"),
        spec_column("org_id"),
        spec_column("root"),
        spec_column("labels"),
        status_column("key_usage_extension"),
        status_column("subject"),
    ]
    return format_table(ctx, certificates, columns)


def list_orgs(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id

    kwargs = strip_none(kwargs)
    return apiclient.trusted_certs_api.list_cert_orgs(
        **kwargs,
    ).orgs


def format_orgs(ctx, orgs):
    columns = [
        column("id"),
        column("organisation"),
    ]
    return format_table(ctx, orgs, columns)



"""
    Agilicus API

    Agilicus is API-first. Modern software is controlled by other software, is open, is available for you to use the way you want, securely, simply.  The OpenAPI Specification in YAML format is available on [www](https://www.agilicus.com/www/api/agilicus-openapi.yaml) for importing to other tools.  A rendered, online viewable and usable version of this specification is available at [api](https://www.agilicus.com/api). You may try the API inline directly in the web page. To do so, first obtain an Authentication Token (the simplest way is to install the Python SDK, and then run `agilicus-cli --issuer https://MYISSUER get-token`). You will need an org-id for most calls (and can obtain from `agilicus-cli --issuer https://MYISSUER list-orgs`). The `MYISSUER` will typically be `auth.MYDOMAIN`, and you will see it as you sign-in to the administrative UI.  This API releases on Bearer-Token authentication. To obtain a valid bearer token you will need to Authenticate to an Issuer with OpenID Connect (a superset of OAUTH2).  Your \"issuer\" will look like https://auth.MYDOMAIN. For example, when you signed-up, if you said \"use my own domain name\" and assigned a CNAME of cloud.example.com, then your issuer would be https://auth.cloud.example.com.  If you selected \"use an Agilicus supplied domain name\", your issuer would look like https://auth.myorg.agilicus.cloud.  For test purposes you can use our [Python SDK](https://pypi.org/project/agilicus/) and run `agilicus-cli --issuer https://auth.MYDOMAIN get-token`.  This API may be used in any language runtime that supports OpenAPI 3.0, or, you may use our [Python SDK](https://pypi.org/project/agilicus/), our [Typescript SDK](https://www.npmjs.com/package/@agilicus/angular), or our [Golang SDK](https://git.agilicus.com/pub/sdk-go).  100% of the activities in our system our API-driven, from our web-admin, through our progressive web applications, to all internals: there is nothing that is not accessible.  For more information, see [developer resources](https://www.agilicus.com/developer).   # noqa: E501

    The version of the OpenAPI document: 2024.09.24
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""


import unittest

import agilicus_api
from agilicus_api.api.rules_api import RulesApi  # noqa: E501


class TestRulesApi(unittest.TestCase):
    """RulesApi unit test stubs"""

    def setUp(self):
        self.api = RulesApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_cleanup_standalone_rules(self):
        """Test case for cleanup_standalone_rules

        cleanup of standalone rules  # noqa: E501
        """
        pass

    def test_create_ruleset_label(self):
        """Test case for create_ruleset_label

        Creates a StandaloneRulesetLabel  # noqa: E501
        """
        pass

    def test_create_standalone_rule(self):
        """Test case for create_standalone_rule

        Add a standalone rule  # noqa: E501
        """
        pass

    def test_create_standalone_rule_policy(self):
        """Test case for create_standalone_rule_policy

        Add a standalone rule policy  # noqa: E501
        """
        pass

    def test_create_standalone_rule_tree(self):
        """Test case for create_standalone_rule_tree

        Add a standalone rule tree  # noqa: E501
        """
        pass

    def test_create_standalone_ruleset(self):
        """Test case for create_standalone_ruleset

        Add a standalone ruleset  # noqa: E501
        """
        pass

    def test_create_standalone_ruleset_bundle(self):
        """Test case for create_standalone_ruleset_bundle

        Add a standalone ruleset bundle  # noqa: E501
        """
        pass

    def test_delete_ruleset_label(self):
        """Test case for delete_ruleset_label

        Delete a StandaloneRulesetLabel  # noqa: E501
        """
        pass

    def test_delete_standalone_rule(self):
        """Test case for delete_standalone_rule

        Delete a standalone rule  # noqa: E501
        """
        pass

    def test_delete_standalone_rule_policy(self):
        """Test case for delete_standalone_rule_policy

        Delete a standalone rule policy  # noqa: E501
        """
        pass

    def test_delete_standalone_rule_tree(self):
        """Test case for delete_standalone_rule_tree

        Delete a standalone rule tree  # noqa: E501
        """
        pass

    def test_delete_standalone_ruleset(self):
        """Test case for delete_standalone_ruleset

        Delete a standalone ruleset  # noqa: E501
        """
        pass

    def test_delete_standalone_ruleset_bundle(self):
        """Test case for delete_standalone_ruleset_bundle

        Delete a standalone ruleset_bundle  # noqa: E501
        """
        pass

    def test_get_ruleset_label(self):
        """Test case for get_ruleset_label

        Get a StandaloneRulesetLabel  # noqa: E501
        """
        pass

    def test_get_standalone_rule(self):
        """Test case for get_standalone_rule

        Get a standalone rule  # noqa: E501
        """
        pass

    def test_get_standalone_rule_policy(self):
        """Test case for get_standalone_rule_policy

        Get a standalone rule policy  # noqa: E501
        """
        pass

    def test_get_standalone_rule_tree(self):
        """Test case for get_standalone_rule_tree

        Get a standalone rule tree  # noqa: E501
        """
        pass

    def test_get_standalone_ruleset(self):
        """Test case for get_standalone_ruleset

        Get a standalone ruleset  # noqa: E501
        """
        pass

    def test_get_standalone_ruleset_bundle(self):
        """Test case for get_standalone_ruleset_bundle

        Get a standalone ruleset bundle  # noqa: E501
        """
        pass

    def test_list_ruleset_labels(self):
        """Test case for list_ruleset_labels

        list StandaloneRulesetLabel  # noqa: E501
        """
        pass

    def test_list_standalone_rule_policies(self):
        """Test case for list_standalone_rule_policies

        List all standalone rule policies  # noqa: E501
        """
        pass

    def test_list_standalone_rule_trees(self):
        """Test case for list_standalone_rule_trees

        List all standalone rule trees  # noqa: E501
        """
        pass

    def test_list_standalone_rules(self):
        """Test case for list_standalone_rules

        List all standalone rules  # noqa: E501
        """
        pass

    def test_list_standalone_ruleset_bundles(self):
        """Test case for list_standalone_ruleset_bundles

        List all standalone ruleset bundles  # noqa: E501
        """
        pass

    def test_list_standalone_rulesets(self):
        """Test case for list_standalone_rulesets

        List all standalone rulesets  # noqa: E501
        """
        pass

    def test_replace_standalone_rule(self):
        """Test case for replace_standalone_rule

        update a standalone rule  # noqa: E501
        """
        pass

    def test_replace_standalone_rule_policy(self):
        """Test case for replace_standalone_rule_policy

        update a standalone rule policy  # noqa: E501
        """
        pass

    def test_replace_standalone_rule_tree(self):
        """Test case for replace_standalone_rule_tree

        update a standalone rule tree  # noqa: E501
        """
        pass

    def test_replace_standalone_ruleset(self):
        """Test case for replace_standalone_ruleset

        update a standalone ruleset  # noqa: E501
        """
        pass

    def test_replace_standalone_ruleset_bundle(self):
        """Test case for replace_standalone_ruleset_bundle

        update a standalone ruleset bundle  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()
